/***********************************************************************************
                         anselm.ru [2014-04-24] GNU GPL
***********************************************************************************/
#pragma once
#include <string>
#include <list>
#include <map>

using namespace std;

class String : public string
{
public:
  String():d_wchar(NULL)   { }
  virtual ~String()        { }
  String(const string &a)    : string(a),d_wchar(NULL) { }
  String(const char* a)      : string(a),d_wchar(NULL) { }
  //String(const int a)      : d_wchar(NULL) { char s[100]; ::sprintf(s, "%d",  a);  *this = s; }
  //String(const long a)     : d_wchar(NULL) { char s[100]; ::sprintf(s, "%ld", a);  *this = s; }
  //String(const float a)    : d_wchar(NULL) { char s[100]; ::sprintf(s, "%f",  a);  *this = s; }
  //String(const double a)   : d_wchar(NULL) { char s[100]; ::sprintf(s, "%lf", a);  *this = s; }
  

  String         to_string(const String& defaults = String(""))    const { return empty() ? defaults : *this; }
  const char*    c_str    (const char*   defaults = "")    const { return empty() ? defaults : string::c_str(); }
  const char*    strdup   (const char*   defaults = "")    const { return ::strdup(c_str(defaults)); }
  int            to_int   (const int     defaults = 0 )    const { return to_long(defaults); }
  long           to_long  (const long    defaults = 0 )    const { 
    if(empty()) return defaults;
    size_t i = find("0x");
    return (i!=String::npos) ? strtol(c_str()+i+2, NULL, 16 ) : strtol(c_str(), NULL, 10 ); 
  }
  float          to_float  (const float   defaults = 0.0)   const { return empty() ? defaults : strtof(c_str(), NULL); }
  double         to_double (const double  defaults = 0.0)   const { return empty() ? defaults : strtod(c_str(), NULL); }
  bool           to_bool   (const bool    defaults = false) const { return empty() ? defaults : (*this=="true" || *this=="yes" || *this=="on" || *this=="1"); }
  timeval        to_timeval() const;
  
  String         hex      (const char* format =   "0x%X")    const { return sprintf(format, to_long()); }
  const wchar_t* utf16    (const char* defaults = "");
  
  //operator short()          const { return to_int(); }
  //operator unsigned short() const { return to_int(); }
  //operator int()            const { return to_int(); }
  //operator unsigned int()   const { return to_int(); }
  //operator unsigned long()  const { return to_long();    }
  
  //operator float()          const { return to_float();   }
  //operator double()         const { return to_double();  }
  //operator bool()           const { return to_bool(); }
  //operator const char*()    const { return string::c_str();  }
  //operator const wchar_t*()       { return utf16(); }

  const char operator [] (int i) const { return string::operator[](i); } // без этого тупит
  
  String&             clear() { string::clear(); return *this; }
  String              trim() const;
  String              bprintf(const String& pattern) const;
  static String sprintf(const String& pattern, ...);
  
// perl-функции
  String&             replace(const String& pattern);
  String&             replace(const char* old, const char* now, const char* flags = "g");
  String              eval() const;  
  bool                match  (const String& pattern) const;
  int                 matches(const String& pattern, list<String>&) const;
  int                 matches(const String& pattern, list<int>&) const;
private:
  static int          bnprintf(const void *data, const size_t size_data,
                        const char *format,
                        char *ret, const size_t size_ret);
  
  static String       perl(const String&);
  wchar_t             *d_wchar;
};

inline bool // для сравнивания String и char*
operator!=(const String& a, const char* b) {// мешает терминальный ноль
  return a.compare(0, strlen(b), b)!=0;
}

inline bool // для сравнивания String и char*
operator==(const String& a, const char* b) {// мешает терминальный ноль
  return a.compare(0, strlen(b), b)==0;
}
